/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#include <cstdio>
#include "misc.hpp"

Param::Param(void) {
    var_max_length = 64;
    precision = 4;

    do_mapping = false;
    dump_mapping = false;
    read_mapping = false;
    dump_routing = false;
    dump_apcg = false;

    // if true, then XY-routing is not necessarily used. Instead, the routing 
    // table will be synthesized to reflect the application requirement
    routing_table_synthesis = false;
    min_hit_threshold = 400;

    dump_mapping_table_file = NULL;
    read_mapping_table_file = NULL;
    routing_table_file = NULL;
    dump_apcg_file = NULL;
    verbose = false;

    link_bandwidth = 1000000;
    buf_read_ebit  = 1.056;    // energy consumption per bit read
    buf_write_ebit = 2.831;    // energy consumption per bit write
    link_ebit = 0.449; 
    switch_ebit = 0.284;
    legal_turn_set = odd_even;
    optimization_method = opt_bbm;
    routing_type = routing_xy;
    routing_effort = easy;

    pq_size = 2000;
    link_usage_matrix = NULL;
    link_usage_list = NULL;

    seed = 5;

    parse_error = false;
    quit_flag = false;
}
