#!/usr/bin/perl -w
#
# Copyright (c) 2005 Carnegie Mellon University.  All rights
# reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:
#       "This product includes software developed by
#        Carnegie Mellon University (http://www.cmu.edu/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. The names "Carnegie Mellon University" and "CMU" must
#    not be used to endorse or promote products derived from this
#    software.
#
# 5. Products derived from this software may not be called
#     "Carnegie Mellon University" or "CMU",
#    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @ProposedRating yellow jingcao@ece.cmu.edu
#
########################################################################

use strict ;

# the first arg is the input apcg file name to be converted
# the second arg is the assumed period

my $apcg_file_name = $ARGV[0];
my $assumed_period = $ARGV[1];

# my $apcg_file_name = "auto-indust-mocsyn-16tile.apcg";
# my $assumed_period = 80000;         # this is to calculate the lamda parameter by dividing volume

my $config_file_name = $apcg_file_name . ".config";

print ("Converting file ", $apcg_file_name, " to ", $config_file_name, " ...\n");

open(in_file, $apcg_file_name) || 
    die("Unable to open file " . $apcg_file_name . " for read.\n");
open(out_file, ">$config_file_name") ||
    die("Unable to open file " . $config_file_name . " for write.\n");


# put header comment in config file
print out_file ("# File automatically generated by utility apcg2config.pl\n",
                "# $apcg_file_name --> $config_file_name\n",
                "# with an assumed application period = $assumed_period\n\n");


while (my $input_line = <in_file>) {
    my @list = split(/[ \t]+/, $input_line);
    if (@list != 4) {
        next;
    }
    (my $first_word) = @list;
    if ( $first_word =~ /^\#/ ) {   # if this is a comment line
        next;
    }
    else {
        ## it should be in the format of (source, destination, volume, bw)
        (my $src, my $dst, my $vol) = @list;
        print out_file ("\@NODE\t", $src, "\n");
        my $lamda = $vol / $assumed_period;
        print out_file ("packet_to_destination_rate\t", $dst, "\t", $lamda, "\n");
        if ( $lamda > 0.25 ) {
            print ("\[W\]: lamda appears to be too large with a value of ", $lamda);
            print (", consider adjust your assumed_period parameter.\n");
        }
    }
}

print ("Succeed.\n");


close(in_file);
close(out_file);
