#!/usr/bin/perl -w
#
# Copyright (c) 2005 Carnegie Mellon University.  All rights
# reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:
#       "This product includes software developed by
#        Carnegie Mellon University (http://www.cmu.edu/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. The names "Carnegie Mellon University" and "CMU" must
#    not be used to endorse or promote products derived from this
#    software.
#
# 5. Products derived from this software may not be called
#     "Carnegie Mellon University" or "CMU",
#    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @ProposedRating yellow jingcao@ece.cmu.edu
#
# Description: A simple perl script which is used to remap a traffic 
# config file.
########################################################################

use strict ;

my $numArgs = $#ARGV + 1;

if ($numArgs != 3) {
    print ("Usage: \nremap_traffic.pl orig_traffic_config map_file new_traffic_config\n");
    exit;
}

my $old_config_file_name = $ARGV[0];
my $map_file_name = $ARGV[1];
my $new_config_file_name = $ARGV[2];

open(old_config_file, $old_config_file_name) ||
    die("Unable to open file " . $old_config_file_name . " for read.\n");

open(map_file, $map_file_name) ||
    die("Unable to open file " . $map_file_name . " for read.\n");

open(new_config_file, ">$new_config_file_name") ||
    die("Unable to open file " . $new_config_file_name . " for write.\n"); 

# map_file is in the format of (for each line)
# old_id     mapped_id
my @new_id = ( 0 );
my $count = 0;

while (my $input_line = <map_file>) {
    if ( $input_line =~ /^\#/ ) {
        next;
    }
    my @id = split(/[ \t]+/, $input_line);
    if ($#id + 1 != 2) {
        die ("Unknown map file format\n");
    }
    my $max = $id[0];
    if ( $max < $id[1] ) {
        $max = $id[1];
    }
    chomp($id[1]);
    if ($max > $#new_id) {
# we need to expand the array
        for ($count = 0; $count <= $max - $#new_id; $count ++) {
            @new_id = (@new_id, -1);
        }
    }
    $new_id[$id[0]] = $id[1];
}

close(map_file);


# now starts remapping, add some heading here

print new_config_file ("# File generated by remapping utility remap_config.pl:\n");
print new_config_file ("# remap_traffic.pl $old_config_file_name $map_file_name $new_config_file_name\n");

while (my $input_line = <old_config_file>) {
    if ( $input_line =~ /^\#/ ) { # if this is a comment line
        next;
    }
    else {
        if ( $input_line =~ /^\@NODE/ ) {
            my @list = split(/[ \t]+/, $input_line);
            (my $node, my $old_id) = @list;
            print new_config_file ("\@NODE\t", $new_id[$old_id], "\n");
        }
        elsif ( $input_line =~ /^packet_to_destination_rate/ ) {
            my @list = split(/[ \t]+/, $input_line);
            (my $semantics, my $old_id, my $rate) = @list;
            print new_config_file ($semantics, "\t", $new_id[$old_id], "\t", $rate);
        }
        else {
            if ( !($input_line =~ /^\n/) ) {
                print ("Unknown semantics\n" . $input_line);
            }
        }
    }
}

close(new_config_file);
close(old_config_file);

