/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#ifndef ARBITER_HPP_
#define ARBITER_HPP_

#include "common.hpp"
#include "misc.hpp"

#include <vector>

#ifdef USE_ORION
extern "C" {
#include "SIM_power.h"
#include "SIM_router_power.h"
#include "SIM_power_router.h"
}
#endif


typedef struct Sw_connection_request{ 
    class Input_channel * channel;
    int timestamp;
    Direction dir;
};
typedef Sw_connection_request *pSw_connection_request;

typedef class Arbiter {
    int delay;                  // the delay it takes to proecess the request
    int num_of_switch_port;     // the switch it controls has how many ports
    class Router * router;

    vector<struct Sw_connection_request *>* request_queue;

    void orion_record_arbiter_power(Direction dir);
    Direction get_current_granted_channel_dir(Direction dir);
    Atom_type form_req_vector(Direction dir) const;

public:
    void tick(void);
    Arbiter(int d, int n_sw_p);
    ~Arbiter();
    void bind(class Router * r) { router = r; }
    void receive_connection_request(class Sw_connection_request * req);

    void process_requests(void);
    bool process_requests_of_port(unsigned int dir);
    void setup_connection(class Input_channel * in, class Output_channel * out);
    void teardown_connection(class Input_channel * in);

    int get_delay(void) const { return delay; }
};
typedef Arbiter *pArbiter;


#endif  // ARBITER_HPP_
