/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 *
 * Description: A port is essentially a bundle of virtual channels,
************************************************************************/

#ifndef PORT_HPP_
#define PORT_HPP_

#include "common.hpp"
#include "misc.hpp"
#include "channel.hpp"

typedef class Port : public Connector { 
protected:
    unsigned int size;
    Direction dir;
    class Router * router;
    vector<class Channel *> channels;
    unsigned int link_user_channel_id;      // which channel in the port is currently using
                                            // the link
public:
    Port(class Router * r, Direction d) { router = r; dir = d; link_user_channel_id = 0; }
    class Channel * get_channel(int index) { return channels[index]; }
    class Channel * get_link_user_channel(void) { return channels[link_user_channel_id]; }
    unsigned int get_size(void) const { return size; }
    unsigned int num_of_empty_buffer_slots(void) const;
    unsigned int total_buffer_capacity(void) const;
    void set_channel_buffer_size(int s);
    Direction get_direction(void) const { return dir; }
    class Router * get_router(void) { return router; }
    ~Port();
    void tick(void);
} Port, *pPort;


typedef class In_port : public Port {
public:
    In_port(class Router * r, Direction d);
    bool can_send(void) const;
    bool can_receive(void) const;
    bool send(void);
    bool receive(class Flit * a_flit);
    void set_routing_scheme(Routing_scheme rs);
    void set_routing_table(char * table_string); // UYO
} In_port, *pIn_port;


typedef class Out_port : public Port {
public:
    Out_port(class Router * r, Direction d);
    bool can_send(void) const;
    bool can_receive(void) const;
    bool send(void);
    bool receive(class Flit * a_flit);
} Out_port, *pOut_port;

#endif  // PORT_HPP_
