/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 *
 * Description: Routing engine is used to decide the output direction of 
               the output channel. Each channel/virtual channel will have 
               one of the engine
************************************************************************/

#ifndef ROUTING_ENGINE_HPP_
#define ROUTING_ENGINE_HPP_

#include "common.hpp"
#include "misc.hpp"


typedef class Routing_engine : public Timer_owner {
protected:
    Position pos;
    Routing_scheme routing_scheme;
    vector<unsigned int> routing_table; // UYO
    class Input_channel * channel;
    int req_time;
    int grant_time;
    int delay_xy;
    int delay_oe;

    Direction dir;      // where to route this packet

    int send_request_to_arbiter(void);    // send the request to arbiter for port connection
    void timer_fired(void);
    Routing_scheme decide_best_routing_mode(void) const;

public:
    Direction decide_direction(const Position & src_pos, const Position & dst_pos) const;
    Direction decide_direction_xy(const Position & src_pos, const Position & dst_pos) const;
    Direction decide_direction_oe(const Position & src_pos, const Position & dst_pos) const;
    Direction decide_direction_oe_fixed(const Position & src_pos, const Position & dst_pos) const;
    Direction decide_direction_oe_split(const Position & src_post, const Position & dst_pos) const;

    vector<Direction> * get_possible_routing_directions(const Position & src_pos, const Position & dst_pos) const;
    vector<Direction> * get_possible_routing_directions_oe(const Position & src_pos, const Position & dst_pos) const;

    void set_routing_scheme(Routing_scheme rs);
    void set_routing_table(char * table_string); // UYO
    void get_request(const class Flit * flit);
    int receive_grant_from_arbiter(void);
    void teardown_connection(void);
    Routing_engine(Position p);
    ~Routing_engine();
    void bind_channel(class Input_channel * chan); 
    int get_delay(void) const;
};

typedef Routing_engine *pRouting_engine;


#endif  //ROUTING_ENGINE_HPP_
